@@ -6,12 +6,42 @@ var fs = require('fs');
 var HttpsProxyAgent = require('https-proxy-agent');
 var Q = require('q');
 var qs = require('qs');
+var url = require('url');
+var https = require('https');
 var Response = require('../http/response');
 var Request = require('../http/request');
 
-axios.defaults.headers.post['Content-Type'] = 'application/x-www-form-urlencoded';
+const DEFAULT_CONTENT_TYPE = 'application/x-www-form-urlencoded';
+const DEFAULT_TIMEOUT = 30000;
 
-var RequestClient = function () { };
+/**
+ * Make http request
+ * @param {object} opts - The options argument
+ * @param {string} opts.timeout - A custom timeout to use. This will be used as the socket timeout, and as the default request timeout.
+ */
+var RequestClient = function (opts) {
+  opts = opts || {};
+  this.defaultTimeout = opts.timeout || DEFAULT_TIMEOUT;
+
+  // construct an https agent
+  let agentOpts = {
+    timeout: this.defaultTimeout,
+  };
+
+  let agent;
+  if (process.env.HTTP_PROXY) {
+    // Note: if process.env.HTTP_PROXY is set, we're not able to apply the given
+    // socket timeout. See: https://github.com/TooTallNate/node-https-proxy-agent/pull/96
+    agent = new HttpsProxyAgent(process.env.HTTP_PROXY);
+  } else {
+    agent = new https.Agent(agentOpts);
+  }
+
+  // construct an axios instance
+  this.axios = axios.create();
+  this.axios.defaults.headers.post['Content-Type'] = DEFAULT_CONTENT_TYPE;
+  this.axios.defaults.httpsAgent = agent;
+};
 
 /**
  * Make http request
@@ -53,12 +83,11 @@ RequestClient.prototype.request = function (opts) {
   }
 
   var options = {
-    timeout: opts.timeout || 30000,
+    timeout: opts.timeout || this.defaultTimeout,
     maxRedirects: opts.allowRedirects ? 10 : 0, // Same number of allowed redirects as request module default
     url: opts.uri,
     method: opts.method,
     headers: opts.headers,
-    httpsAgent: process.env.HTTP_PROXY ? new HttpsProxyAgent(process.env.HTTP_PROXY) : undefined,
     proxy: false,
     validateStatus: status => status >= 100 && status < 600,
   };
@@ -99,7 +128,7 @@ RequestClient.prototype.request = function (opts) {
   this.lastResponse = undefined;
   this.lastRequest = new Request(optionsRequest);
 
-  axios(options).then((response) => {
+  this.axios(options).then((response) => {
     if (opts.logLevel === 'debug') {
       console.log(`response.statusCode: ${response.status}`)
       console.log(`response.headers: ${JSON.stringify(response.headers)}`)
@@ -122,7 +151,7 @@ RequestClient.prototype.filterLoggingHeaders = function (headers){
   return Object.keys(headers).filter((header) => {
     return !'authorization'.includes(header.toLowerCase());
   });
-}
+};
 
 RequestClient.prototype.logRequest = function (options){
   console.log('-- BEGIN Twilio API Request --');
@@ -140,6 +169,6 @@ RequestClient.prototype.logRequest = function (options){
   }
 
   console.log('-- END Twilio API Request --');
-}
+};
 
 module.exports = RequestClient;
